@@ -1894,6 +1894,12 @@ bool IsPcdSupported()
 	UINT type;
 	OS_INFO *info = GetOsInfo();
 
+	if (MsIsWindows10())
+	{
+		// Windows 10 or later never supports PCD driver.
+		return false;
+	}
+
 	type = info->OsType;
 
 	if (OS_IS_WINDOWS_NT(type) == false)
@@ -1972,11 +1978,6 @@ HINSTANCE InstallPcdDriverInternal()
 			src_filename = BRIDGE_WIN32_PCD_SYS_X64;
 		}
 
-		if (MsIsIA64())
-		{
-			src_filename = BRIDGE_WIN32_PCD_SYS_IA64;
-		}
-
 		// Copy see.sys
 		if (FileCopy(src_filename, tmp) == false)
 		{
@@ -1995,10 +1996,6 @@ HINSTANCE InstallPcdDriverInternal()
 		{
 			dll_filename = BRIDGE_WIN32_PCD_DLL_X64;
 		}
-		else if (MsIsIA64())
-		{
-			dll_filename = BRIDGE_WIN32_PCD_DLL_IA64;
-		}
 	}
 
 	// Try to load see.dll and initialize
